@@ -1,4 +1,5 @@
-"""MINUIT from Python - Fitting like a boss
+"""
+MINUIT2 from Python
 
 Basic usage example::
 
@@ -12,23 +13,16 @@ Basic usage example::
 
 Further information:
 
-* Code: https://github.com/iminuit/iminuit
+* Code: https://github.com/scikit-hep/iminuit
 * Docs: https://iminuit.readthedocs.io
 """
 
-__all__ = [
-    'Minuit',
-    'minimize',
-    'describe',
-    'Struct',
-    '__version__',
-    'test',
-]
+__all__ = ["Minuit", "minimize", "describe", "__version__", "test"]
 
 from ._libiminuit import Minuit
 from ._minimize import minimize
-from .util import describe, Struct
-from .info import __version__
+from .util import describe
+from .version import __version__
 
 
 def test(args=None):
@@ -38,9 +32,10 @@ def test(args=None):
 
     From the command line:
 
-        python -c 'import iminuit; iminuit.test()
+        python -c 'import iminuit; iminuit.test()'
     """
     # http://pytest.org/latest/usage.html#calling-pytest-from-python-code
     import pytest
-    args = ['-v', '--pyargs', 'iminuit']
+
+    args = ["-v", "--pyargs", "iminuit"]
     pytest.main(args)
