@@ -1,6 +1,4 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-import random
+from __future__ import absolute_import, division, print_function, unicode_literals
 from math import sqrt, exp, cos, pi, e
 from iminuit import Minuit
 from iminuit.tests.utils import assert_allclose
@@ -12,13 +10,12 @@ def rosenbrock(x, y):
 
 
 def test_rosenbrock():
-    random.seed(0.258)
     m = Minuit(rosenbrock, x=0, y=0, pedantic=False, print_level=1)
     m.tol = 1e-4
     m.migrad()
     assert_allclose(m.fval, 0, atol=1e-6)
-    assert_allclose(m.values['x'], 1., atol=1e-3)
-    assert_allclose(m.values['y'], 1., atol=1e-3)
+    assert_allclose(m.values["x"], 1.0, atol=1e-3)
+    assert_allclose(m.values["y"], 1.0, atol=1e-3)
 
 
 def ackleys(x, y):
@@ -28,11 +25,9 @@ def ackleys(x, y):
 
 
 def test_ackleys():
-    random.seed(0.258)
-
-    m = Minuit(ackleys, x=1.5 * random.random(), y=1.5 * random.random(),
-               error_x=1.7, error_y=1.7,
-               pedantic=False, print_level=0)
+    m = Minuit(
+        ackleys, x=0.3, y=-0.2, error_x=1.7, error_y=1.7, pedantic=False, print_level=0
+    )
     m.tol = 1e-4
     m.migrad()
 
@@ -49,9 +44,7 @@ def beale(x, y):
 
 
 def test_beale():
-    random.seed(0.258)
-    m = Minuit(beale, x=random.random(), y=0.5 * random.random(),
-               pedantic=False, print_level=0)
+    m = Minuit(beale, x=0.5, y=0.25, pedantic=False, print_level=0)
     m.tol = 1e-4
     m.migrad()
     assert_allclose(m.fval, 0, atol=1e-6)
@@ -64,9 +57,7 @@ def matyas(x, y):
 
 
 def test_matyas():
-    random.seed(0.258)
-    m = Minuit(matyas, x=random.random(), y=random.random(),
-               pedantic=False, print_level=0)
+    m = Minuit(matyas, x=0.5, y=0.5, pedantic=False, print_level=0)
     m.tol = 1e-4
     m.migrad()
 
@@ -76,10 +67,9 @@ def test_matyas():
 
 def test_matyas_oneside():
     """One-sided limit when the minimum is in the forbidden region"""
-    random.seed(0.258)
-    m = Minuit(matyas, x=2 + random.random(), y=random.random(),
-               limit_x=(1, None),
-               pedantic=False, print_level=0)
+    m = Minuit(
+        matyas, x=2 + 0.5, y=0.5, limit_x=(1, None), pedantic=False, print_level=0
+    )
     m.tol = 1e-4
     m.migrad()
     assert_allclose(m.args, [1, 0.923], atol=1e-3)
