@@ -1,5 +1,4 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
+from __future__ import absolute_import, division, print_function, unicode_literals
 from iminuit import minimize
 from iminuit.tests.utils import assert_allclose, requires_dependency
 import numpy as np
@@ -14,7 +13,7 @@ def grad(x, *args):
     return 2 * (x - (0, 1, 2))
 
 
-@requires_dependency('scipy')
+@requires_dependency("scipy")
 def test_simple():
     result = minimize(func, (1, 1, 1))
     assert_allclose(result.x, (0, 1, 2), atol=1e-8)
@@ -23,7 +22,7 @@ def test_simple():
     assert result.njev == 0
 
 
-@requires_dependency('scipy')
+@requires_dependency("scipy")
 def test_gradient():
     result = minimize(func, (1, 1, 1), jac=grad)
     assert_allclose(result.x, (0, 1, 2), atol=1e-8)
@@ -32,7 +31,7 @@ def test_gradient():
     assert result.njev > 0
 
 
-@requires_dependency('scipy')
+@requires_dependency("scipy")
 def test_args():
     result = minimize(func, np.ones(3), args=(5,))
     assert_allclose(result.x, (0, 1, 2), atol=1e-8)
@@ -41,11 +40,10 @@ def test_args():
     assert result.njev == 0
 
 
-@requires_dependency('scipy')
+@requires_dependency("scipy")
 def test_callback():
     trace = []
-    result = minimize(func, np.ones(3),
-                      callback=lambda x: trace.append(x.copy()))
+    result = minimize(func, np.ones(3), callback=lambda x: trace.append(x.copy()))
     assert_allclose(result.x, (0, 1, 2), atol=1e-8)
     assert_allclose(result.fun, 1)
     assert result.nfev == len(trace)
